<?php

namespace Drupal\doi_search\Controller;

use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Form\FormBuilderInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Drupal\doi_search\DoiSearchManager;

/**
 * Returns responses for DOI Publications routes.
 */
class DoiSearchController extends ControllerBase {

  /**
   * The form builder.
   *
   * @var \Drupal\Core\Form\FormBuilderInterface
   */
  protected $formBuilder;

  /**
   * The request stack.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * The Doi search manager.
   *
   * @var Drupal\doi_search\DoiSearchManager
   */
  protected $doiSearchManager;

  /**
   * The controller constructor.
   *
   * @param \Drupal\Core\Form\FormBuilderInterface $form_builder
   *   The form builder.
   */
  public function __construct(FormBuilderInterface $form_builder,RequestStack $request,DoiSearchManager $doiSearchManager) {
    $this->formBuilder = $form_builder;
    $this->request = $request;
    $this->doiSearchManager = $doiSearchManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('form_builder'),
      $container->get('request_stack'),
      $container->get('doi_search.manager')
    );
  }

  /**
   * Builds the response.
   */
  public function build() {
    $form = $this->formBuilder->getForm('\Drupal\doi_search\Form\SearchForm');
    $doi = $this->request->getCurrentRequest()->query->get('doi');
    $info = [];
    if (!empty($doi)){
      $data = $this->doiSearchManager->getData($doi);
      if (isset($data->DOI)){
        $data->formatedAuthors = $this->formatAuthors($data->author);
        $data->pdf = $this->getPdfLink($data);
        $info = [
          "#theme" => 'doi_search_result',
          '#items' => (array) $data
        ];
        $references = $this->getPublicationReferences($data);
      }
    }
    $build = [
      '#theme' => 'doi_search_page',
      '#items' => [
        'form' => $form,
        'result' =>  $info,
        'references' => isset($references) ? $references : NULL,
        'empty' => empty($doi)
      ],
    ];
    return $build;
  }

  /**
   * It takes a DOI and returns an array of reference data
   * 
   * @param doi The DOI of the reference you want to retrieve.
   * 
   * @return An array of reference data.
   */
  private function getReferenceData($doi){
    if ($doi){
      $data = (array) $this->doiSearchManager->getData($doi);
      if (!empty($data)){
        $date = (array) $data['created'];
        $refData = [
          'title' => $this->getPublicationTitle($data),
          'author' => isset($data['author']) ? $this->formatAuthors($data['author']) : NULL,
          'year' => date("d M Y",strtotime($date['date-time'])),
          'doi' => isset($data['DOI']) ? $data['DOI'] : '',
          'url' => isset($data['URL']) ? $data['URL'] : ''
        ];
      }
    }
    return isset($refData) ? $refData : NULL;
  }

  /**
   * It loops through the array of data and returns the title of the publication
   * 
   * @param data The data array from the JSON response.
   * 
   * @return The title of the publication.
   */
  private function getPublicationTitle($data){
    $title = '';
    foreach ($data as $key => $value){
      if(strpos($key, "title") !== false && !empty($value) && !(strpos($key, "container") !== false)){
       $title = is_array($value) ? $value[0] : $value;
       break;
      }
    }
    return $title;
  }

  /**
   * If the link is a PDF, return the link. Otherwise, return NULL
   * 
   * @param data the data object returned from the API
   * 
   * @return the pdf link if it exists.
   */
  private function getPdfLink($data){
    if (isset($data->link)){
      foreach ($data->link as $link){
        if(strtolower(substr($link->URL, -3, 3)) == "pdf"){
          $pdf = $link->URL;
          break;
        }             
      }
    }
    return isset($pdf) ? $pdf : NULL;
  }

  /**
   * It takes a data object, loops through the references, and returns an array of
   * references to render
   * 
   * @param data the data returned from the API
   * 
   * @return An array of references.
   */
  private function getPublicationReferences($data){
    $references = [];
    if (isset($data->reference)){
      foreach ($data->reference as $reference){
        $refData = [];
        if (isset($reference->DOI)){
          $refData = $this->getReferenceData($reference->DOI);
        }
        else {
          $refArray = (array) $reference;
          $refData = [
            'title' => $this->getPublicationTitle($refArray),
            'author' => isset($refArray['author']) ? $refArray['author'] : '',
            'year' => isset($refArray['year']) ? $refArray['year'] : '',
            'doi' => isset($refArray['DOI']) ? $refArray['DOI'] : ''
          ];
        }
        $references[] = [
          '#theme' => 'doi_search_reference',
          '#items' => $refData
        ];
      }
    }
    return $references;
  }

  /**
   * It takes an array of authors and returns a string of authors
   * 
   * @param authors The authors of the article.
   * 
   * @return the formatted authors.
   */
  private function formatAuthors($authors){
    $formatedAuthors = [];
    foreach ($authors as $author){
      $given = isset($author->given) ? $author->given.' ' : '';
      if (empty($given) && !isset($author->family)){
        return isset($author->name) ? $author->name : '';
      }
      $formatedAuthors[] =  $given.$author->family;
    }
    return implode(', ',$formatedAuthors);
  }
}
